@@ -1,7 +1,9 @@
 use std::convert::TryInto;
 
+use serde::de::{
+    self, DeserializeSeed, SeqAccess, VariantAccess, IntoDeserializer, Visitor
+};
 use serde::Deserialize;
-use serde::de::{self, DeserializeSeed, SeqAccess, VariantAccess, Visitor};
 
 use crate::{Error, Result};
 
@@ -41,16 +43,17 @@ impl<'de> Deserializer<'de> {
     }
 
     fn next_bytes_const<const SIZE: usize>(&mut self) -> Result<[u8; SIZE]> {
-        let bytes: [u8; SIZE] = 
-            self.input[..SIZE]
-                .try_into()
-                .map_err(|_| Error::Eof)?;
-        self.input = &self.input[SIZE..];
-        Ok(bytes)
+        if self.input.len() < SIZE {
+            Err(Error::Eof)
+        } else {
+            let bytes: [u8; SIZE] = self.input[..SIZE].try_into().map_err(|_| Error::Eof)?;
+            self.input = &self.input[SIZE..];
+            Ok(bytes)
+        }
     }
 
     fn next_bytes(&mut self, size: usize) -> Result<&'de [u8]> {
-        if self.input.len() > size {
+        if self.input.len() < size {
             Err(Error::Eof)
         } else {
             let bytes = &self.input[..size];
@@ -78,7 +81,7 @@ macro_rules! impl_for_deserialize_primitive {
         {
             visitor.$visitor_fname(<$type>::from_be_bytes(self.next_bytes_const()?))
         }
-    }
+    };
 }
 
 impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
@@ -168,22 +171,14 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         visitor.visit_unit()
     }
 
-    fn deserialize_unit_struct<V>(
-        self,
-        _name: &'static str,
-        visitor: V,
-    ) -> Result<V::Value>
+    fn deserialize_unit_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
         self.deserialize_unit(visitor)
     }
 
-    fn deserialize_newtype_struct<V>(
-        self,
-        _name: &'static str,
-        visitor: V,
-    ) -> Result<V::Value>
+    fn deserialize_newtype_struct<V>(self, _name: &'static str, visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
@@ -199,8 +194,7 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
             len: usize,
         }
 
-        impl<'a, 'de> SeqAccess<'de> for Access<'a, 'de>
-        {
+        impl<'a, 'de> SeqAccess<'de> for Access<'a, 'de> {
             type Error = Error;
 
             fn next_element_seed<T>(&mut self, seed: T) -> Result<Option<T::Value>>
@@ -251,23 +245,37 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         self.deserialize_tuple(fields.len(), visitor)
     }
 
-    #[cfg(feature = "is_human_readable")]
-    /// Always return `false`
-    fn is_human_readable(&self) -> bool {
-        false
-    }
-
-    /// Unsupported
     fn deserialize_enum<V>(
         self,
         _name: &'static str,
         _variants: &'static [&'static str],
-        _visitor: V,
+        visitor: V,
     ) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
-        Err(Error::Unsupported("serialize_variant"))
+        impl<'a, 'de> serde::de::EnumAccess<'de> for &'a mut Deserializer<'de>
+        {
+            type Error = Error;
+            type Variant = Self;
+
+            fn variant_seed<V>(self, seed: V) -> Result<(V::Value, Self::Variant)>
+            where
+                V: de::DeserializeSeed<'de>,
+            {
+                let idx: u32 = self.next_u32()?;
+                let val: Result<_> = seed.deserialize(idx.into_deserializer());
+                Ok((val?, self))
+            }
+        }
+
+        visitor.visit_enum(self)
+    }
+
+    #[cfg(feature = "is_human_readable")]
+    /// Always return `false`
+    fn is_human_readable(&self) -> bool {
+        false
     }
 
     /// Unsupported
@@ -319,7 +327,6 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
     }
 }
 
-
 /// Unsupported
 impl<'a, 'de> VariantAccess<'de> for &'a mut Deserializer<'de> {
     type Error = Error;
@@ -346,8 +353,7 @@ impl<'a, 'de> VariantAccess<'de> for &'a mut Deserializer<'de> {
     }
 
     /// Unsupported
-    fn struct_variant<V>(self, _fields: &'static [&'static str], _visitor: V)
-        -> Result<V::Value>
+    fn struct_variant<V>(self, _fields: &'static [&'static str], _visitor: V) -> Result<V::Value>
     where
         V: Visitor<'de>,
     {
@@ -360,13 +366,15 @@ impl<'a, 'de> VariantAccess<'de> for &'a mut Deserializer<'de> {
 mod tests {
     use super::*;
     use crate::*;
-    use serde::{Serialize, de::DeserializeOwned};
+    use assert_matches::assert_matches;
+    use serde::{de::DeserializeOwned, Serialize};
     use std::fmt::Debug;
 
     /// First serialize value, then deserialize it.
     fn test_roundtrip<T: Debug + Eq + Serialize + DeserializeOwned>(value: &T) {
         let serialized = to_bytes(value).unwrap();
-        assert_eq!(from_bytes::<T>(&serialized).unwrap(), *value);
+        // Ignore the size
+        assert_eq!(from_bytes::<T>(&serialized[4..]).unwrap(), *value);
     }
 
     #[test]
@@ -387,7 +395,8 @@ mod tests {
     fn test_str() {
         let s = "Hello, world!";
         let serialized = to_bytes(&s).unwrap();
-        let deserialized: &str = from_bytes(&serialized).unwrap();
+        // Ignore the size
+        let deserialized: &str = from_bytes(&serialized[4..]).unwrap();
         assert_eq!(deserialized, s);
     }
 
@@ -418,4 +427,17 @@ mod tests {
             v4: 0x7812,
         });
     }
+
+    /// Test EOF error
+    #[test]
+    fn test_eof_error() {
+        assert_matches!(from_bytes::<u8>(&[]), Err(Error::Eof));
+
+        let s = "Hello, world!";
+        let serialized = to_bytes(&s).unwrap();
+        assert_matches!(
+            from_bytes::<String>(&serialized[0..serialized.len() - 1]),
+            Err(Error::Eof)
+        );
+    }
 }
