@@ -1,34 +1,52 @@
-use std::convert::TryInto;
 use serde::{ser, Serialize};
+use std::convert::TryInto;
 
 use crate::{Error, Result};
 
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct Serializer {
     output: Vec<u8>,
 }
 
+impl Default for Serializer {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Serializer {
     pub fn new() -> Self {
-        Self::default()
+        Self {
+            output: [0, 0, 0, 0].into(),
+        }
     }
 
-    pub fn get_output(&self) -> &[u8] {
-        &self.output
+    /// Return a byte array with the first 4 bytes representing the size
+    /// of the rest of the serialized message.
+    pub fn get_output(&mut self) -> Result<&[u8]> {
+        let len: u32 = (self.output.len() - 4)
+            .try_into()
+            .map_err(|_| Error::BytesTooLong)?;
+        self.output[..4].copy_from_slice(&len.to_be_bytes());
+        Ok(&self.output)
     }
 
     /// Clear the output but preserve its allocated memory
     pub fn reset(&mut self) {
-        self.output.clear();
+        self.output.truncate(4);
     }
 }
 
+/// Return a byte array with the first 4 bytes representing the size
+/// of the rest of the serialized message.
 pub fn to_bytes<T>(value: &T) -> Result<Vec<u8>>
 where
     T: Serialize,
 {
     let mut serializer = Serializer::default();
     value.serialize(&mut serializer)?;
+    // Fill the first 4 bytes with the size
+    serializer.get_output()?;
     Ok(serializer.output)
 }
 
@@ -38,7 +56,7 @@ macro_rules! impl_for_serialize_primitive {
             self.output.extend_from_slice(&v.to_be_bytes());
             Ok(())
         }
-    }
+    };
 }
 
 impl<'a> ser::Serializer for &'a mut Serializer {
@@ -87,9 +105,7 @@ impl<'a> ser::Serializer for &'a mut Serializer {
     }
 
     fn serialize_bytes(self, v: &[u8]) -> Result<()> {
-        let len: u32 = v.len()
-            .try_into()
-            .map_err(|_| Error::BytesTooLong)?;
+        let len: u32 = v.len().try_into().map_err(|_| Error::BytesTooLong)?;
 
         self.serialize_u32(len)?;
 
@@ -117,11 +133,7 @@ impl<'a> ser::Serializer for &'a mut Serializer {
         self.serialize_unit()
     }
 
-    fn serialize_newtype_struct<T>(
-        self,
-        _name: &'static str,
-        value: &T,
-    ) -> Result<()>
+    fn serialize_newtype_struct<T>(self, _name: &'static str, value: &T) -> Result<()>
     where
         T: ?Sized + Serialize,
     {
@@ -144,11 +156,53 @@ impl<'a> ser::Serializer for &'a mut Serializer {
         self.serialize_tuple(len)
     }
 
-    fn serialize_struct(
+    fn serialize_struct(self, _name: &'static str, len: usize) -> Result<Self::SerializeStruct> {
+        self.serialize_tuple(len)
+    }
+
+    fn serialize_unit_variant(
         self,
         _name: &'static str,
+        variant_index: u32,
+        _variant: &'static str,
+    ) -> Result<()> {
+        self.serialize_u32(variant_index)
+    }
+
+    fn serialize_newtype_variant<T>(
+        self,
+        name: &'static str,
+        variant_index: u32,
+        variant: &'static str,
+        value: &T,
+    ) -> Result<()>
+    where
+        T: ?Sized + Serialize,
+    {
+        self.serialize_unit_variant(name, variant_index, variant)?;
+        value.serialize(self)
+    }
+
+    fn serialize_tuple_variant(
+        self,
+        name: &'static str,
+        variant_index: u32,
+        variant: &'static str,
         len: usize,
-    ) -> Result<Self::SerializeStruct> {
+    ) -> Result<Self::SerializeTupleVariant> {
+        self.serialize_unit_variant(name, variant_index, variant)?;
+        self.serialize_tuple(len)
+    }
+
+    /// Unsupported
+    fn serialize_struct_variant(
+        self,
+        name: &'static str,
+        variant_index: u32,
+        variant: &'static str,
+        len: usize,
+    ) -> Result<Self::SerializeStructVariant> {
+        self.serialize_unit_variant(name, variant_index, variant)?;
         self.serialize_tuple(len)
     }
 
@@ -162,52 +216,6 @@ impl<'a> ser::Serializer for &'a mut Serializer {
     fn serialize_map(self, _len: Option<usize>) -> Result<Self::SerializeMap> {
         Err(Error::Unsupported("serialize_map"))
     }
-
-    /// Unsupported
-    fn serialize_unit_variant(
-        self,
-        _name: &'static str,
-        _variant_index: u32,
-        _variant: &'static str,
-    ) -> Result<()> {
-        Err(Error::Unsupported("serialize_variant"))
-    }
-
-    /// Unsupported
-    fn serialize_newtype_variant<T>(
-        self,
-        _name: &'static str,
-        _variant_index: u32,
-        _variant: &'static str,
-        _value: &T,
-    ) -> Result<()>
-    where
-        T: ?Sized + Serialize,
-    {
-        Err(Error::Unsupported("serialize_variant"))
-    }
-
-    /// Unsupported
-    fn serialize_tuple_variant(
-        self,
-        _name: &'static str,
-        _variant_index: u32,
-        _variant: &'static str,
-        _len: usize,
-    ) -> Result<Self::SerializeTupleVariant> {
-        Err(Error::Unsupported("serialize_variant"))
-    }
-
-    /// Unsupported
-    fn serialize_struct_variant(
-        self,
-        _name: &'static str,
-        _variant_index: u32,
-        _variant: &'static str,
-        _len: usize,
-    ) -> Result<Self::SerializeStructVariant> {
-        Err(Error::Unsupported("serialize_variant"))
-    }
 }
 
 macro_rules! impl_serialize_trait {
@@ -215,24 +223,25 @@ macro_rules! impl_serialize_trait {
         impl<'a> ser::$name for &'a mut Serializer {
             type Ok = ();
             type Error = Error;
-        
+
             fn $function_name<T>(&mut self, value: &T) -> Result<()>
             where
                 T: ?Sized + Serialize,
             {
                 value.serialize(&mut **self)
             }
-        
+
             fn end(self) -> Result<()> {
                 Ok(())
             }
         }
-    }
+    };
 }
 
 impl_serialize_trait!(SerializeSeq, serialize_element);
 impl_serialize_trait!(SerializeTuple, serialize_element);
 impl_serialize_trait!(SerializeTupleStruct, serialize_field);
+impl_serialize_trait!(SerializeTupleVariant, serialize_field);
 
 /// Unsupported
 impl<'a> ser::SerializeMap for &'a mut Serializer {
@@ -276,82 +285,69 @@ impl<'a> ser::SerializeStruct for &'a mut Serializer {
         Ok(())
     }
 }
-
-/// Unsupported
 impl<'a> ser::SerializeStructVariant for &'a mut Serializer {
     type Ok = ();
     type Error = Error;
 
-    /// Unsupported
-    fn serialize_field<T>(&mut self, _key: &'static str, _value: &T) -> Result<()>
+    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<()>
     where
         T: ?Sized + Serialize,
     {
-        Err(Error::Unsupported("serialize_variant"))
+        ser::SerializeStruct::serialize_field(self, key, value)
     }
 
-    /// Unsupported
     fn end(self) -> Result<()> {
-        Err(Error::Unsupported("serialize_variant"))
-    }
-}
-
-/// Unsupported
-impl<'a> ser::SerializeTupleVariant for &'a mut Serializer {
-    type Ok = ();
-    type Error = Error;
-
-    /// Unsupported
-    fn serialize_field<T>(&mut self, _value: &T) -> Result<()>
-    where
-        T: ?Sized + Serialize,
-    {
-        Err(Error::Unsupported("serialize_variant"))
-    }
-
-    /// Unsupported
-    fn end(self) -> Result<()> {
-        Err(Error::Unsupported("serialize_variant"))
+        ser::SerializeStruct::end(self)
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use serde::ser;
+
+    type SerializerToTest = Serializer;
 
     #[test]
     fn test_integer() {
-        assert_eq!(to_bytes(&0x12_u8).unwrap(), [0x12]);
-        assert_eq!(to_bytes(&0x1234_u16).unwrap(), [0x12, 0x34]);
-        assert_eq!(to_bytes(&0x12345678_u32).unwrap(), [0x12, 0x34, 0x56, 0x78]);
+        assert_eq!(to_bytes(&0x12_u8).unwrap(), [0, 0, 0, 1, 0x12]);
+        assert_eq!(to_bytes(&0x1234_u16).unwrap(), [0, 0, 0, 2, 0x12, 0x34]);
+        assert_eq!(
+            to_bytes(&0x12345678_u32).unwrap(),
+            [0, 0, 0, 4, 0x12, 0x34, 0x56, 0x78]
+        );
         assert_eq!(
             to_bytes(&0x1234567887654321_u64).unwrap(),
-            [0x12, 0x34, 0x56, 0x78, 0x87, 0x65, 0x43, 0x21]
+            [0, 0, 0, 8, 0x12, 0x34, 0x56, 0x78, 0x87, 0x65, 0x43, 0x21]
         );
     }
 
     #[test]
     fn test_boolean() {
-        assert_eq!(to_bytes(&true).unwrap(), [0, 0, 0, 1]);
-        assert_eq!(to_bytes(&false).unwrap(), [0, 0, 0, 0]);
+        assert_eq!(to_bytes(&true).unwrap(), [0, 0, 0, 4, 0, 0, 0, 1]);
+        assert_eq!(to_bytes(&false).unwrap(), [0, 0, 0, 4, 0, 0, 0, 0]);
     }
 
     #[test]
     fn test_str() {
         let s = "Hello, world!";
         let serialized = to_bytes(&s).unwrap();
-        assert_eq!(&serialized[..4], to_bytes(&(s.len() as u32)).unwrap());
-        assert_eq!(&serialized[4..], s.as_bytes());
+        let len: u32 = (serialized.len() - 4).try_into().unwrap();
+        assert_eq!(&serialized[..4], len.to_be_bytes());
+        assert_eq!(&serialized[4..8], (s.len() as u32).to_be_bytes());
+        assert_eq!(&serialized[8..], s.as_bytes());
     }
 
     #[test]
     fn test_array() {
         let array = [0x00_u8, 0x01_u8, 0x10_u8, 0x78_u8];
-        assert_eq!(to_bytes(&array).unwrap(), array);
+        let serialized = to_bytes(&array).unwrap();
+        assert_eq!(serialized[..4], [0, 0, 0, 4]);
+        assert_eq!(serialized[4..], array);
 
         assert_eq!(
             to_bytes(&[0x0010_u16, 0x0100_u16, 0x1034_u16, 0x7812_u16]).unwrap(),
-            &[0x00_u8, 0x10_u8, 0x01_u8, 0x00_u8, 0x10_u8, 0x34_u8, 0x78_u8, 0x12_u8]
+            &[0, 0, 0, 8, 0x00, 0x10, 0x01, 0x00, 0x10, 0x34, 0x78, 0x12_u8]
         );
     }
 
@@ -359,7 +355,7 @@ mod tests {
     fn test_tuple() {
         assert_eq!(
             to_bytes(&(0x00_u8, 0x0100_u16, 0x1034_u16, 0x7812_u16)).unwrap(),
-            &[0x00_u8, 0x01_u8, 0x00_u8, 0x10_u8, 0x34_u8, 0x78_u8, 0x12_u8]
+            &[0, 0, 0, 7, 0x00_u8, 0x01_u8, 0x00_u8, 0x10_u8, 0x34_u8, 0x78_u8, 0x12_u8]
         );
     }
 
@@ -380,7 +376,24 @@ mod tests {
         };
         assert_eq!(
             to_bytes(&v).unwrap(),
-            &[0x00_u8, 0x01_u8, 0x00_u8, 0x10_u8, 0x34_u8, 0x78_u8, 0x12_u8]
+            &[0, 0, 0, 7, 0x00_u8, 0x01_u8, 0x00_u8, 0x10_u8, 0x34_u8, 0x78_u8, 0x12_u8]
+        );
+    }
+
+    #[test]
+    fn test_enum() {
+        use ser::Serializer;
+
+        let mut serializer = SerializerToTest::new();
+
+        serializer.serialize_unit_variant("", 1, "").unwrap();
+        assert_eq!(serializer.get_output().unwrap(), [0, 0, 0, 4, 0, 0, 0, 1]);
+
+        serializer.reset();
+        serializer.serialize_newtype_variant("", 0, "", &3).unwrap();
+        assert_eq!(
+            serializer.get_output().unwrap(),
+            [0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 3]
         );
     }
 }
